@@ -63,7 +63,7 @@ make_local_tmpdir (const char *dirname)
 				g_error ("Can not stat %s\n", dirname);
 
 #if !defined (__CYGWIN__) && !defined(_WIN32)
-			if (statbuf.st_uid != getuid ())
+			if (getuid() != 0 && statbuf.st_uid != getuid ())
 				g_error ("Owner of %s is not the current user\n", dirname);
 
 			if ((statbuf.st_mode & (S_IRWXG|S_IRWXO)) ||
@@ -139,11 +139,46 @@ link_get_tmpdir (void)
 
 #if defined(AF_INET) || defined(AF_INET6) || defined (AF_UNIX)
 
-/* hopefully a sufficiently large default NIC count */
-#ifdef DEFAULT_LINC2_IFNUM
-#undef DEFAULT_LINC2_IFNUM
+#ifndef G_OS_WIN32
+static void
+get_first_non_local_ipaddr(char *buf, 
+			   size_t len)
+{
+	int sock;
+	const char *retv;
+	struct if_nameindex *ifname_idx_array;
+	struct if_nameindex *ifname_idx;
+	struct sockaddr_in sin;
+	struct ifreq ifr;
+
+	*buf = '\0';
+
+	sock = socket(AF_INET, SOCK_DGRAM, 0);
+	if (-1 == sock)
+		return;
+
+	ifname_idx_array = if_nameindex();
+	for (ifname_idx = ifname_idx_array; ifname_idx && ifname_idx->if_name && ifname_idx->if_name[0]; ifname_idx++) {
+		strncpy(ifr.ifr_name, ifname_idx->if_name, IFNAMSIZ);
+		if (ioctl(sock, SIOCGIFADDR, &ifr)) 
+			continue;
+
+		memcpy(&sin, &(ifr.ifr_addr), sizeof(struct sockaddr_in));
+		retv = (const char*)inet_ntoa(sin.sin_addr);
+		retv = strcmp("127.0.0.1", retv) ? retv : NULL;
+		if (retv) {
+			strncpy(buf, (const char*)inet_ntoa(sin.sin_addr), len);
+			break;
+		}
+	}
+
+	if (ifname_idx_array)
+		if_freenameindex(ifname_idx_array);
+
+	if (-1 != sock)
+		close(sock);
+}
 #endif
-#define DEFAULT_LINC2_IFNUM (32)
 
 static char *
 get_netid(LinkNetIdType which, 
@@ -161,58 +196,13 @@ get_netid(LinkNetIdType which,
 	if ((LINK_NET_ID_IS_IPADDR == which)
 	    || (LINK_NET_ID_IS_CUSTOM == which)) {
 #ifndef G_OS_WIN32
-		struct sockaddr_in *adr = NULL;
-		struct ifreq *my_ifreqs = NULL;
-		struct ifconf my_ifconf = { 0 };
-		int sock = -1;
-		int num = 0;
-		int i = 0;
+		if (fixed_host_net_id) 
+			strncpy(buf, fixed_host_net_id, len);
+		else
+			get_first_non_local_ipaddr(buf, len);
+		if ('\0' == *buf)
+			strncpy(buf, "127.0.0.1", len);
 
-		sock = socket(AF_INET,SOCK_DGRAM,0);
-		if (-1 == sock) 
-			goto out;
-#ifdef SIOCGIFNUM
-		i = ioctl(sock, SIOCGIFNUM, &num);
-		if ((0 > i) || !num) {
-			close(sock);
-			goto out;
-		}
-#else
-		num = DEFAULT_LINC2_IFNUM;
-#endif
-		my_ifconf.ifc_len = sizeof(struct ifreq) * num;
-		my_ifconf.ifc_req = (struct ifreq*)malloc(my_ifconf.ifc_len);
-		if (!my_ifconf.ifc_req) {
-			close(sock);
-			goto out;
-		}
-		if (ioctl(sock,SIOCGIFCONF,&my_ifconf) < 0) {
-			close(sock);
-			free(my_ifconf.ifc_req);
-			goto out;
-		}
-		close(sock);
-
-		my_ifreqs = my_ifconf.ifc_req;
- 		for (i = 0; i < num; i++) {
-			adr = (struct sockaddr_in *)&my_ifreqs[i].ifr_ifru.ifru_addr;
-			if (fixed_host_net_id) {
-				if (!strcmp(fixed_host_net_id, inet_ntoa(adr->sin_addr)))
-					break;
-			} else {
-				if (strcmp("127.0.0.1", inet_ntoa(adr->sin_addr)))
-					break;
-			}
-			if (!strcmp("0.0.0.0", inet_ntoa(adr->sin_addr))) {
-				if (i) 
-					adr = (struct sockaddr_in *)&my_ifreqs[--i].ifr_ifru.ifru_addr;
-				break;
-			}
-		}
-		strncpy(buf, (const char*)inet_ntoa(adr->sin_addr), len);
-		free(my_ifconf.ifc_req);
-
-		/* will be 127.0.0.1 anyway, if no other address is defined... */
 		return buf;
 #else
 		SOCKET sock;
@@ -220,10 +210,8 @@ get_netid(LinkNetIdType which,
 		/* Let's hope 20 interfaces is enough. There doesn't
 		 * seem to be any way to get information about how
 		 * many interfaces there are.
-		 * 
-		 * [Jules] Using the ifnum define instead...
 		 */
-		INTERFACE_INFO interfaces[DEFAULT_LINC2_IFNUM]; 
+		INTERFACE_INFO interfaces[20]; 
 		int i;
 
 		sock = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
@@ -1006,6 +994,28 @@ link_protocol_get_sockinfo_unix (const LinkProtocolInfo  *proto,
 
 	return TRUE;
 }
+
+/*
+ * link_protocol_post_create_unix:
+ * @fd: the file descriptor.
+ * @sockaddr: a #sockaddr_un structure describing the socket.
+ *
+ * For a user running a program as root, set the owner of the socket to
+ * the original user.
+ */
+static void
+link_protocol_post_create_unix (int fd, struct sockaddr *saddr)
+{
+#ifndef G_OS_WIN32
+  struct sockaddr_un *saddr_un = (struct sockaddr_un *)saddr;
+
+	if (getuid() == 0) {
+		struct stat stat_buf;
+		if (!stat (link_tmpdir, &stat_buf))
+			fchown (fd, stat_buf.st_uid, -1);
+	}
+#endif
+}
 #endif /* AF_UNIX */
 
 /*
@@ -1143,7 +1153,8 @@ static LinkProtocolInfo static_link_protocols[] = {
 	NULL, 				/* destroy */
 	link_protocol_get_sockaddr_ipv4,/* get_sockaddr */
 	link_protocol_get_sockinfo_ipv4,/* get_sockinfo */
-	link_protocol_is_local_ipv46    /* is_local */
+	link_protocol_is_local_ipv46,   /* is_local */
+	NULL				/* post_create */
 	},
 #endif
 #if defined(AF_INET6)
@@ -1157,7 +1168,8 @@ static LinkProtocolInfo static_link_protocols[] = {
 	NULL, 				/* destroy */
 	link_protocol_get_sockaddr_ipv6,/* get_sockaddr */
 	link_protocol_get_sockinfo_ipv6,/* get_sockinfo */
-	link_protocol_is_local_ipv46    /* is_local */
+	link_protocol_is_local_ipv46,   /* is_local */
+	NULL				/* post_create */
 	},
 #endif
 #ifdef AF_UNIX
@@ -1171,7 +1183,8 @@ static LinkProtocolInfo static_link_protocols[] = {
 	link_protocol_unix_destroy,  			/* destroy */
 	link_protocol_get_sockaddr_unix, 		/* get_sockaddr */
 	link_protocol_get_sockinfo_unix, 		/* get_sockinfo */
-	link_protocol_unix_is_local                     /* is_local */
+	link_protocol_unix_is_local,                     /* is_local */
+	link_protocol_post_create_unix			/* post_create */
 	},
 #endif
 	{ NULL /* name */ }
